/*
 * Copyright (c) 2010 Ryan Flannery <ryan.flannery@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef CONFIG_H
#define CONFIG_H

const int DEFAULT_LIBRARY_WINDOW_WIDTH = 18;

/* XXX IMPORTANT NOTE:
 * If you're having problems with the keybindings on your platform, most
 * likely it's because the key-mappings below between "CONTROL+x" are
 * different on your system.  I'm trying to find a better way of mapping
 * these, but ncurses(3) does not appear to be able to handle these (via
 * keypad(3) or any other mechanism) in some reasonable fashion.
 */
#define K_TAB      9
#define K_ENTER    13
#define K_CNTRL_B  2
#define K_CNTRL_C  3
#define K_CNTRL_D  4
#define K_CNTRL_E  5
#define K_CNTRL_F  6
#define K_CNTRL_L  12
#define K_CNTRL_P  16
#define K_CNTRL_U  21
#define K_CNTRL_Y  25
#define K_CNTRL_SLASH 28


/* list of keybindings */
const keybinding KeyBindings[] = {
   { K_CNTRL_C,      quit_vitunes,     { 0 } },
   { K_CNTRL_SLASH,  quit_vitunes,     { 0 } },

   { K_ENTER,        load_or_play,     { 0 } },
   { 'm',            show_file_info,   { 0 } },
   { K_CNTRL_P,      pause_playback,   { 0 } },
   { 'z',            pause_playback,   { 0 } },
   { 's',            stop_playback,    { 0 } },
   { '[',            seek_playback,    { .direction = Backwards, .scale = Seconds, .amount = 10 }},
   { ']',            seek_playback,    { .direction = Forwards,  .scale = Seconds, .amount = 10 }},
   { 'b',            seek_playback,    { .direction = Backwards, .scale = Seconds, .amount = 10 }},
   { 'f',            seek_playback,    { .direction = Forwards,  .scale = Seconds, .amount = 10 }},
   { '{',            seek_playback,    { .direction = Backwards, .scale = Minutes, .amount = 1 }},
   { '}',            seek_playback,    { .direction = Forwards,  .scale = Minutes, .amount = 1 }},
   { 'B',            seek_playback,    { .direction = Backwards, .scale = Minutes, .amount = 1 }},
   { 'F',            seek_playback,    { .direction = Forwards,  .scale = Minutes, .amount = 1 }},

   { K_TAB,          switch_focus,     { 0 } },
   { K_CNTRL_L,      redraw,           { 0 } },

   { 'j',            scroll_row,       { .direction = Down }},
   { KEY_DOWN,       scroll_row,       { .direction = Down }},
   { 'k',            scroll_row,       { .direction = Up }},
   { KEY_UP,         scroll_row,       { .direction = Up }},
   { '-',            scroll_row,       { .direction = Up }},

   { 'h',            scroll_col,       { .direction = Left,  .amount = One }},
   { KEY_LEFT,       scroll_col,       { .direction = Left,  .amount = One }},
   { KEY_BACKSPACE,  scroll_col,       { .direction = Left,  .amount = One }},
   { 'l',            scroll_col,       { .direction = Right, .amount = One }},
   { KEY_RIGHT,      scroll_col,       { .direction = Right, .amount = One }},
   { ' ',            scroll_col,       { .direction = Right, .amount = One }},
   { '$',            scroll_col,       { .direction = Right, .amount = Whole }},
   { '^',            scroll_col,       { .direction = Left,  .amount = Whole }},
   { '0',            scroll_col,       { .direction = Left,  .amount = Whole }},

   { K_CNTRL_E,      scroll_page,      { .direction = Down, .amount = One }},
   { K_CNTRL_Y,      scroll_page,      { .direction = Up,   .amount = One }},
   { K_CNTRL_U,      scroll_page,      { .direction = Up,   .amount = Half }},
   { K_CNTRL_D,      scroll_page,      { .direction = Down, .amount = Half }},
   { K_CNTRL_B,      scroll_page,      { .direction = Up,   .amount = Whole }},
   { KEY_PPAGE,      scroll_page,      { .direction = Up,   .amount = Whole }},
   { K_CNTRL_F,      scroll_page,      { .direction = Down, .amount = Whole }},
   { KEY_NPAGE,      scroll_page,      { .direction = Down, .amount = Whole }},

   { 'G',            jumpto_file,      { .amount = Number }},
   { '%',            jumpto_file,      { .amount = Percent }},

   { 'H',            jumpto_page,      { .location = Top }},
   { 'M',            jumpto_page,      { .location = Middle }},
   { 'L',            jumpto_page,      { .location = Bottom }},

   { 'y',            yank,             { 0 } },
   { 'p',            paste,            { .location = After }},
   { 'P',            paste,            { .location = Before }},
   { 'd',            cut,              { 0 } },

   { '/',            search,           { .direction = Forwards }},
   { '?',            search,           { .direction = Backwards }},
   { 'n',            search_find,      { .direction = Same }},
   { 'N',            search_find,      { .direction = Opposite }},

   { ':',            enter_cmd_mode,   { 0 }}
};

const int KeyBindingsSize = (sizeof(KeyBindings) / sizeof(keybinding));
#define ExecuteKeyBinding(i)  \
   ((KeyBindings[(i)].func)(KeyBindings[(i)].args))


/* list of command-mode commands */
const cmd CommandPath[] = {
   {  "q",        cmd_quit },
   {  "q!",       cmd_quit },
   {  "w",        cmd_write },
   {  "w!",       cmd_write },
   {  "mode",     cmd_mode },
   {  "new",      cmd_new },
   {  "filter",   cmd_filter },
   {  "filter!",  cmd_filter },
   {  "sort",     cmd_sort },
   {  "display",  cmd_display },
   {  "color",    cmd_color },
   {  "set",      cmd_set },
   {  "reload",   cmd_reload }
};

const int CommandPathSize = (sizeof(CommandPath) / sizeof(cmd));

#endif
