/*
 * Copyright (c) 2010 Ryan Flannery <ryan.flannery@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef UINTERFACE_H
#define UINTERFACE_H

#include <sys/ioctl.h>
#include <sys/ioctl_compat.h>
#include <ncurses.h>
#include <ctype.h>
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <termios.h>
#include <unistd.h>
#include <err.h>

#include "debug.h"

typedef enum { Up, Down, Left, Right } Direction;

/* return codes used for various things */
enum { UI_OK, UI_USER_CANCELLED };

/* struct for a scrollable window */
typedef struct
{
   WINDOW  *cwin;
   int      w, h;    /* it's just handy to track these here */
   int      crow;    /* current row in the window */
   int      nrows;   /* number of rows in the window */
   int      voffset; /* vertical & horizontal scroll offsets */
   int      hoffset;

} swindow;

#define CMD_ROW_FMT_SIZE 256


/* user itnerface */
typedef struct
{
   WINDOW   *player;
   WINDOW   *command;
   char     cmd_row_fmt[CMD_ROW_FMT_SIZE];
      /* used to right-align text in command window */

   swindow  *library;
   swindow  *playlist;

   /* this is always a pointer to one of the two above swindow's */
   swindow  *active;

   int  lwidth; /* width of the library pane on the left */
   bool lhide;  /* if library window should be hidden or not */

} uinterface;

extern uinterface ui;   /* the global ui struct */


void ui_init(int library_width);
void ui_destroy();
void ui_resize();

void ui_hide_library();
void ui_unhide_library();

swindow *swindow_new(int h, int w, int y, int x);
void swindow_free(swindow *win);
void swindow_resize(swindow *win, int h, int w, int y, int x);
void swindow_scroll(swindow *win, Direction d, int n);

void ui_draw_borders();
void ui_clear();

/* input methods */
int  ui_command_get_string(const char *prompt, char **input);
int  ui_command_get_yesno(const char *prompt, int *response);

#endif

